==> 0.0.2a.zip: menagerie_cookbook/shared/src/commonMain/kotlin/recipeslist/RecipesList.kt <==
package recipeslist

import androidx.compose.animation.ExperimentalSharedTransitionApi
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.WindowInsets
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.systemBars
import androidx.compose.foundation.layout.windowInsetsPadding
import androidx.compose.foundation.lazy.grid.GridCells
import androidx.compose.foundation.lazy.grid.LazyGridState
import androidx.compose.foundation.lazy.grid.LazyVerticalGrid
import androidx.compose.foundation.lazy.grid.rememberLazyGridState
import androidx.compose.material3.Card
import androidx.compose.material3.Text
import androidx.compose.runtime.*
import androidx.compose.ui.Modifier
import model.Recipe


/**
 * Created by abdulbasit on 25/06/2023.
 */

@OptIn(ExperimentalSharedTransitionApi::class)
@Composable
fun RecipesListScreen(
    items: List<Recipe>,
    onClick: (recipe: Recipe) -> Unit,
    isLarge: Boolean,
) {
    Box(
        modifier = Modifier.fillMaxSize()
    ) {
        val listState = rememberLazyGridState()
        if (items.isEmpty()) {
            Card { Text("NO RECIPE FOUND") }
        }
        else {
            LazyVerticalGrid(
                state = listState, columns = GridCells.Fixed(if (isLarge) 3 else 1)
            ) {
                if (isLarge.not())
                    item {
                        Spacer(modifier = Modifier.windowInsetsPadding(WindowInsets.systemBars))
                    }
                items(items.size) { item ->
                    val recipe = items[item]
                    RecipeListItemWrapper(
                        scrollDirection = listState.isScrollingUp(),
                        child = {
                            RecipeListItem(
                                recipe = recipe,
                                onClick = onClick,
                            )
                        }
                    )
                }
            }
        }
    }
}

@Composable
private fun LazyGridState.isScrollingUp(): Boolean {
    var previousIndex by remember(this) { mutableStateOf(firstVisibleItemIndex) }
    var previousScrollOffset by remember(this) { mutableStateOf(firstVisibleItemScrollOffset) }
    return remember(this) {
        derivedStateOf {
            if (previousIndex != firstVisibleItemIndex) {
                previousIndex > firstVisibleItemIndex
            } else {
                previousScrollOffset >= firstVisibleItemScrollOffset
            }.also {
                previousIndex = firstVisibleItemIndex
                previousScrollOffset = firstVisibleItemScrollOffset
            }
        }
    }.value
}

==> 0.1.0a.zip: menagerie_cookbook/shared/src/commonMain/kotlin/recipeslist/RecipesList.kt <==
package recipeslist

import androidx.compose.animation.ExperimentalSharedTransitionApi
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.WindowInsets
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.systemBars
import androidx.compose.foundation.layout.windowInsetsPadding
import androidx.compose.foundation.lazy.grid.GridCells
import androidx.compose.foundation.lazy.grid.LazyGridState
import androidx.compose.foundation.lazy.grid.LazyVerticalGrid
import androidx.compose.foundation.lazy.grid.rememberLazyGridState
import androidx.compose.material3.Card
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.runtime.*
import androidx.compose.ui.Modifier
import model.Recipe


/**
 * Created by abdulbasit on 25/06/2023.
 */

@OptIn(ExperimentalSharedTransitionApi::class)
@Composable
fun RecipesListScreen(
    items: List<Recipe>,
    onClick: (recipe: Recipe) -> Unit,
    isLarge: Boolean,
) {
    Box(
        modifier = Modifier.fillMaxSize()
    ) {
        val listState = rememberLazyGridState()
        if (items.isEmpty()) {
            Card { Text("NO RECIPE FOUND") }
        }
        else {

            val columns = if (isLarge) 3 else 1

            LazyVerticalGrid(
                state = listState, columns = GridCells.Fixed(columns)
            ) {
                if (isLarge.not())
                    item {
                        Spacer(modifier = Modifier.windowInsetsPadding(WindowInsets.systemBars))
                    }
                items(items.size) { item ->
                    val recipe = items[item]
                    recipe.bgColor = (if(item % 2 == 0) MaterialTheme.colorScheme.primaryContainer else MaterialTheme.colorScheme.secondaryContainer)
                    RecipeListItemWrapper(
                        scrollDirection = listState.isScrollingUp(),
                        child = {
                            RecipeListItem(
                                recipe = recipe,
                                onClick = onClick,
                            )
                        }
                    )
                }
            }
        }
    }
}

@Composable
private fun LazyGridState.isScrollingUp(): Boolean {
    var previousIndex by remember(this) { mutableStateOf(firstVisibleItemIndex) }
    var previousScrollOffset by remember(this) { mutableStateOf(firstVisibleItemScrollOffset) }
    return remember(this) {
        derivedStateOf {
            if (previousIndex != firstVisibleItemIndex) {
                previousIndex > firstVisibleItemIndex
            } else {
                previousScrollOffset >= firstVisibleItemScrollOffset
            }.also {
                previousIndex = firstVisibleItemIndex
                previousScrollOffset = firstVisibleItemScrollOffset
            }
        }
    }.value
}
